@@ -1,4 +1,4 @@
-################################### Paquetes ##############################################
+############################### Paquetes ##############################################
 import pandas as pd
 from scipy import stats
 import numpy as np
@@ -11,14 +11,34 @@ from sklearn.preprocessing import StandardScaler
 from sklearn import linear_model
 from sklearn.model_selection import cross_val_score, cross_val_predict
 import joblib
+from sklearn.linear_model import ElasticNet
 from sklearn.ensemble import RandomForestRegressor
 from sklearn.model_selection import GridSearchCV
 import sys
 import xgboost as xgb
+from IPython.display import display
+from sklearn import metrics
+from sklearn.cluster import KMeans 
+from sklearn.cluster import DBSCAN
+from sklearn.neighbors import NearestNeighbors
+from kneed import KneeLocator
+from sklearn.decomposition import PCA
+from sklearn.feature_selection import mutual_info_classif
+import folium
+import leafmap.foliumap as leafmap
+from IPython.display import display
+from sklearn import metrics
+from sklearn.preprocessing import StandardScaler
+from sklearn.cluster import DBSCAN
+from kneed import KneeLocator
+from sklearn.neighbors import NearestNeighbors
+import matplotlib.animation as animation
+
 
 #################################### Funciones ##############################################
 
 
+
 # Función para agregar columnas con indices (ufm2, etc) y cambiar el tipo de
 # dato de ciertas columnas
 def InsertColumns(D): #(D es la tabla) 
@@ -362,23 +382,75 @@ def Muestra_uni(df1,df2,cve,ant):
                         on=["ingreso","antiguedad","longitud","latitud","supCSII","supTSII","valor","avaluofiscal"])
     MuestraAVM=datosduplicados(MuestraAVM,False)
     return MuestraML,MuestraAVM
-#def Muestra_Total(A):
-#    if A=="ML":
-#        LR = pd.read_excel('MuestraML19.xlsx');
-#        LC = pd.read_excel('MuestraML21.xlsx');
-#        LB = pd.read_excel('MuestraML22.xlsx');
-#        V = pd.read_excel('MuestraML52.xlsx');
-#        Muestra = pd.concat([LR,LC,LB,V], ignore_index=True, sort=False)
-#        Muestra.to_excel('Muestra_Total_ML.xlsx')
-#    elif A=="AVM":
-#        LR = pd.read_excel('MuestraAVM19.xlsx');
-#        LC = pd.read_excel('MuestraAVM21.xlsx');
-#        LB = pd.read_excel('MuestraAVM22.xlsx');
-#        V = pd.read_excel('MuestraAVM52.xlsx');
-#        Muestra = pd.concat([LR,LC,LB,V], ignore_index=True, sort=False)
-#        Muestra.to_excel('Muestra_Total_AVM.xlsx')
-#    return Muestra
 
+def DelUbicaciones(df):
+    L=df.loc[:,'longitud'].tolist()
+    L2=df.loc[:,'latitud'].tolist()
+    # Contar y guardar decimales de longitud
+    digitosLongitud=[]
+    for k in range(len(L)):
+        c=1
+        sLk=str(L[k])
+        for s in range(len(sLk)):
+            if sLk[s]!='.':
+                c=c+1
+            else:
+                break
+        dk=len(str(L[k]))-c
+        digitosLongitud.append(dk)
+    df.insert(df.shape[1],'D_longitud',digitosLongitud, True)
+    # Contar y guardar decimales de latitud
+    digitosLatitud=[]
+    for k in range(len(L2)):
+        c=1
+        sLk=str(L2[k])
+        for s in range(len(sLk)):
+            if sLk[s]!='.':
+                c=c+1
+            else:
+                break
+        dk=len(str(L2[k]))-c
+        digitosLatitud.append(dk)
+    df.insert(df.shape[1],'D_latitud',digitosLatitud, True)
+    # quedarnos solo con las ubicaciones con más de 6 decimales
+    df=df[(df['D_longitud']>=6)&(df['D_latitud']>=6)]
+    df=df[['ingreso','antiguedad','longitud','latitud','supCSII','supTSII','avaluofiscal','valor','supTSII/supCSII',
+           'cve_comuna','ufm2(supCSII)','ufm2(supTSII)']]
+    return df
+def grafico_geografico_leafmap(df,df1,df2,defcolor1,defcolor2,defcolor3,ancho,alto):
+    map_geo = leafmap.Map(center=[df['latitud'].mean(),df['longitud'].mean()],zoom=8,
+                         width=ancho, height=alto)
+    
+    map_geo.add_circle_markers_from_xy(df,x="longitud", y="latitud",radius=5, color=defcolor1, fill_color=defcolor1,
+                                           popup=["longitud","latitud","valor"])
+    
+    map_geo.add_circle_markers_from_xy(df1,x="longitud", y="latitud",radius=5, color=defcolor2, fill_color=defcolor2,
+                                           popup=["longitud","latitud","valor"])
+    
+    map_geo.add_circle_markers_from_xy(df2,x="longitud", y="latitud",radius=1, color=defcolor3, fill_color=defcolor3,
+                                          popup=["longitud","latitud","valor"])
+    
+    return display(map_geo)
+def agrupamiento_dbscan(X,pdf,R,Min):
+    # Compute DBSCAN
+    db = DBSCAN(eps=R, min_samples=Min).fit(X)
+    core_samples_mask = np.zeros_like(db.labels_, dtype=bool)
+    core_samples_mask[db.core_sample_indices_] = True
+    labels = db.labels_
+    new_pdf=pdf.assign(Clus_Db=labels)
+
+    realClusterNum=len(set(labels)) - (1 if -1 in labels else 0)
+    clusterNum = len(set(labels)) 
+
+    # Number of data in a cluster
+    Number_of_data=[];
+    for clust_number in set(labels):
+        if (clust_number!=-1):
+                clust_set = new_pdf[new_pdf.Clus_Db == clust_number];
+                Number_of_data.append(clust_set.shape[0]);
+
+    return [labels,realClusterNum,Number_of_data,new_pdf]
+    
 def indexacion(df,mini,maxi):
     years1=[]
     for i in range(mini,maxi,10):
@@ -612,6 +684,8 @@ def Tabla_Estadistica_Error(y31,y32,y41,y42): #### MOD POR TENER RL Y ENET
         tabla=pd.concat([tabla, tablaux],ignore_index=True);
     return tabla
 
+
+
 ##################### PARTE 3 #########################
 
 def InsertColumnasAVM(reavm):
